package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.math.BigDecimal;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.client.api.constant.PatientDataDimensionTypeEnum;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.PatientDiseaseDataOneDayInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.PatientDiseaseDataOneDayInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientDiseaseStatisticQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientDiseaseStatisticVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientDiseaseDataQueryBizService;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class PatientDiseaseDataQueryBizServiceImpl implements PatientDiseaseDataQueryBizService {

    @Autowired
    private PatientDiseaseDataOneDayInfoService patientDiseaseDataOneDayInfoService;

    @Override
    public PatientDiseaseStatisticVO patientStatistic(PatientDiseaseStatisticQueryDTO params) {
        log.info("PatientDataQueryBizServiceImpl.patientStatistic:params >>>>>> {}", JSON.toJSONString(params));

        PatientDiseaseStatisticVO vo = new PatientDiseaseStatisticVO();
        
        // 发现潜在患者/未就诊
        PatientDiseaseDataOneDayInfo noVisitCount = this.getPatientDiseaseDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_Pre_Not_Visit);
        vo.setNotVisitCount(noVisitCount.getNum());
        vo.setNotVisitCountYesterday(noVisitCount.getAddNum());

        // 发现潜在患者/已就诊
        PatientDiseaseDataOneDayInfo visitedCount = this.getPatientDiseaseDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_Pre_Visited);
        vo.setVisitedCount(visitedCount.getNum());
        vo.setVisitedCountYesterday(visitedCount.getAddNum());

        // 发现潜在患者/已确诊
        PatientDiseaseDataOneDayInfo confirmedCount = this.getPatientDiseaseDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_Pre_Confirmed);
        vo.setConfirmedCount(confirmedCount.getNum());
        vo.setConfirmedCountYesterday(confirmedCount.getAddNum());

        // 就诊转化率: (发现潜在患者/已就诊) / (发现潜在患者/未就诊) * 100%
        String transformRatio = vo.getNotVisitCount() == 0 ? "0%" : new BigDecimal(vo.getVisitedCount() + "").divide(new BigDecimal(vo.getNotVisitCount() + ""), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal("100")) + "%";
        log.info("PatientDataQueryBizServiceImpl.patientStatistic:transformRatio >>>>>> {}", JSON.toJSONString(transformRatio));
        vo.setTransformRatio(transformRatio);

        // 近30天就诊转化率
        BigDecimal notVisitedCount30 = this.getTransformRatio30(params, PatientDataDimensionTypeEnum.LSD_Pre_Not_Visit);
        BigDecimal visitedCount30 = this.getTransformRatio30(params, PatientDataDimensionTypeEnum.LSD_Pre_Visited);
        String transformRatio30 = notVisitedCount30.intValue() == 0 ? "0%" : visitedCount30.divide(notVisitedCount30, 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal("100")) + "%";
        log.info("PatientDataQueryBizServiceImpl.patientStatistic:transformRatio30 >>>>>> {}", JSON.toJSONString(transformRatio30));
        vo.setTransformRatio30(transformRatio30);

        // 挂号流失患者
        // 统计本科室对应所有疾病对应的标签分组为“挂号流失患者”所有标签的患者总数量。根据userId去重
        // 挂号流失患者的数据定义为“预约挂号”且“7天内没有该科室就诊记录”的患者
        PatientDiseaseDataOneDayInfo registerCount = this.getPatientDiseaseDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_During_Register);
        vo.setRegisterCount(registerCount.getNum());
        vo.setRegisterCountYesterday(registerCount.getAddNum());

        // 住院流失患者
        // 统计本科室对应所有疾病对应的标签分组为“住院流失患者”所有标签的患者总数量。根据userId去重
        // 住院流失患者的数据定义为“提交住院申请单”且“7天内住院就诊记录”的患者
        PatientDiseaseDataOneDayInfo inHospitalCount = this.getPatientDiseaseDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_During_InHospital);
        vo.setInHospitalCount(inHospitalCount.getNum());
        vo.setInHospitalCountYesterday(inHospitalCount.getAddNum());

        // 手术流失患者
        // 统计本科室对应所有疾病对应的标签分组为“手术流失患者”所有标签的患者总数量。根据userId去重
        // 手术流失患者的数据定义为“申请手术”且“7天内没该手术记录”的患者
        PatientDiseaseDataOneDayInfo operationCount = this.getPatientDiseaseDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_During_Operation);
        vo.setOperationCount(operationCount.getNum());
        vo.setOperationCountYesterday(operationCount.getAddNum());

        // 流失比例
        // 分子：所有申请单；分母：标签人数
        // 累计人数在此处为流失比例，
        PatientDiseaseDataOneDayInfo lostRatio = this.getPatientDiseaseDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_During_Lost);
        vo.setLostRatio(lostRatio.getNum() + "%");
        vo.setLostRatioYesterday(lostRatio.getAddNum());

        // // 疾病管理服务次数
        // PatientDiseaseDataOneDayInfo serviceCount = this.getPatientDiseaseDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_After_Service);
        // vo.setServiceCount(serviceCount.getNum());
        // vo.setServiceCountYesterday(serviceCount.getAddNum());

        // // 服务接受率
        // PatientDiseaseDataOneDayInfo serviceRatio = this.getPatientDiseaseDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_After_Accept);
        // vo.setServiceRatio(serviceRatio.getNum() + "%");
        // vo.setServiceRatioYesterday(serviceRatio.getAddNum() + "%");

        // // 复诊率
        // PatientDiseaseDataOneDayInfo consultationRatio = this.getPatientDiseaseDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_After_Consultation);
        // vo.setConsultationRatio(consultationRatio.getNum() + "%");
        // vo.setConsultationRatioYesterday(consultationRatio.getAddNum() + "%");

        // // 失访人数
        // PatientDiseaseDataOneDayInfo lostCount = this.getPatientDiseaseDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_After_Lost);
        // vo.setLostCount(lostCount.getNum());
        // vo.setLostCountYesterday(lostCount.getAddNum());

        return vo;
    }

    private PatientDiseaseDataOneDayInfo getPatientDiseaseDataOneDayInfo(PatientDiseaseStatisticQueryDTO params, PatientDataDimensionTypeEnum type) {
        log.info("PatientDiseaseDataQueryBizServiceImpl.getPatientDiseaseDataOneDayInfo:params:type >>>>>> {} : {}", JSON.toJSONString(params), type.getCode());
        ValidatorUtils.validateEntity(params);

        PatientDiseaseDataOneDayInfo endDateEntity = patientDiseaseDataOneDayInfoService.getOne(new QueryWrapper<PatientDiseaseDataOneDayInfo>()
                .eq("dept_id", params.getDeptId())
                .eq("disease_id", params.getDiseaseId())
                .eq("data_dimension", type.getCode())
                .eq(StrUtil.isNotBlank(params.getEndDate()), "biz_day", params.getEndDate())
                .orderByDesc("biz_day")
                .last(" limit 1 ")
        );
        // 查询开始时间的数据
        if (StrUtil.isNotBlank(params.getStartDate()) && params.getStartDate().compareTo(endDateEntity.getBizDay()) < 0) {
            PatientDiseaseDataOneDayInfo startDateEntity = patientDiseaseDataOneDayInfoService.getOne(new QueryWrapper<PatientDiseaseDataOneDayInfo>()
                .eq("dept_id", params.getDeptId())
                .eq("disease_id", params.getDiseaseId())
                .eq("data_dimension", type.getCode())
                .eq("biz_day", params.getStartDate())
                // .orderByDesc("biz_day")
                .last(" limit 1 ")
            );
            if (null != startDateEntity) {
                endDateEntity.setNum(endDateEntity.getNum() - startDateEntity.getNum());
                endDateEntity.setAddNum(endDateEntity.getAddNum() - startDateEntity.getAddNum());
            }
        }

        if (null == endDateEntity) {
            endDateEntity = new PatientDiseaseDataOneDayInfo();
            endDateEntity.setNum(0);
            endDateEntity.setAddNum(0);
        }

        log.info("PatientDiseaseDataQueryBizServiceImpl.getPatientDiseaseDataOneDayInfo:result <<<<<< {}", JSON.toJSONString(endDateEntity));
        return endDateEntity;
    }
    
    private BigDecimal getTransformRatio30(PatientDiseaseStatisticQueryDTO params, PatientDataDimensionTypeEnum type) {
        log.info("PatientDiseaseDataQueryBizServiceImpl.getTransformRatio30:params:type >>>>>> {} : {}", JSON.toJSONString(params), type.getCode());
        ValidatorUtils.validateEntity(params);

        DateTime now = DateUtil.date();
        String before30 = DateUtil.offsetDay(now, -30).toDateStr();

        // TODO 先查询最近一天的数据的biz_day，再根据该时间查询当天所有的病种数据

        PatientDiseaseDataOneDayInfo endDateEntity = patientDiseaseDataOneDayInfoService.getOne(new QueryWrapper<PatientDiseaseDataOneDayInfo>()
                .eq("dept_id", params.getDeptId())
                .eq("disease_id", params.getDiseaseId())
                .eq("data_dimension", type.getCode())
                // .eq("biz_day", DateUtil.offsetDay(now, -30))
                .orderByDesc("biz_day")
                .last(" limit 1 ")
        );

        BigDecimal count = new BigDecimal(0);
        if (null == endDateEntity || endDateEntity.getBizDay().compareTo(before30) < 0) {
            return count;
        }
        count = new BigDecimal(endDateEntity.getNum());

        PatientDiseaseDataOneDayInfo startDateEntity = patientDiseaseDataOneDayInfoService.getOne(new QueryWrapper<PatientDiseaseDataOneDayInfo>()
                .eq("dept_id", params.getDeptId())
                .eq("disease_id", params.getDiseaseId())
                .eq("data_dimension", type.getCode())
                .eq("biz_day", DateUtil.offsetDay(now, -30))
                .orderByDesc("biz_day")
        );

        if (null != startDateEntity) {
            count = count.subtract(new BigDecimal(startDateEntity.getNum()));
        }

        return count;
    }

}
